/*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*/
#include "PWMPi.h"

// Pointers to the hardware register bases
volatile uint32_t *PWM_gpio = (volatile uint32_t *)MAP_FAILED;
volatile uint32_t *PWM_pwm  = (volatile uint32_t *)MAP_FAILED;
volatile uint32_t *PWM_clk  = (volatile uint32_t *)MAP_FAILED;

/*******************************
 *                             *
 * PWMPi Class implementation *
 * --------------------------- *
 *******************************/

/******************
 * Public methods *
 ******************/

//Constructor 
PWMPi::PWMPi(){
}

//Initiate the PWM library and set PWM0 (GPIO18)
void PWMPi::begin(){
  int memfd = -1;
  
  // Open the master /dev/memory device
  if ((memfd = open("/dev/mem", O_RDWR | O_SYNC) ) < 0) 
  {
    printf("Failed to open /dev/mem\n");
    // return -1;
  }
  
  PWM_gpio = (volatile uint32_t *)PWM_mapmem("gpio", BLOCK_SIZE, memfd, GPIO_BASE);
  if (PWM_gpio == MAP_FAILED) printf("Failed to map the physical GPIO registers into virtual memory space.\n");
  
  PWM_pwm = (volatile uint32_t *)PWM_mapmem("pwm", BLOCK_SIZE, memfd, PWM_BASE);
  if (PWM_pwm == MAP_FAILED) printf("Failed to map the physical PWM registers into virtual memory space.\n");
  
  // Clock control (needed for PWM)
  PWM_clk = (volatile uint32_t *)PWM_mapmem("clk", BLOCK_SIZE, memfd, CLK_BASE);
  if (PWM_clk == MAP_FAILED) printf("Failed to map the physical CLK registers into virtual memory space.\n");
  
  // Set the output pin to Alt Fun 5, to allow PWM channel 0 to be output there
  bcm2835_PWM_gpio_fsel(18, BCM2835_PWM_GPIO_FSEL_ALT5);
  
  // init default values
  setClockDivider(PWM_CLOCK_DIVIDER_16);
  setMode( 1, 1);
  setRange( 1024 ); 
}

//Initiate the PWM library and set PWM0 (GPIO18)
void PWMPi::close(){
  // Set the output pin from Alt Fun 5 to Input
  bcm2835_PWM_gpio_fsel(18, BCM2835_PWM_GPIO_FSEL_INPT);
  PWM_unmapmem((void**) &PWM_gpio, BLOCK_SIZE);
  PWM_unmapmem((void**) &PWM_pwm,  BLOCK_SIZE);
  PWM_unmapmem((void**) &PWM_clk,  BLOCK_SIZE);
}

//Set the clock divider and enable PWM clock
void PWMPi::setClockDivider(uint32_t divisor)
{
  // From Gerts code
  divisor &= 0xfff;
  // Stop PWM clock
  bcm2835_PWM_peri_write(PWM_clk + BCM2835_PWMCLK_CNTL, BCM2835_PWM_PASSWRD | 0x01);
  delay(110); // Prevents clock going slow
  // Wait for the clock to be not busy
  while ((bcm2835_PWM_peri_read(PWM_clk + BCM2835_PWMCLK_CNTL) & 0x80) != 0)
    delay(1); 
  // set the clock divider and enable PWM clock
  bcm2835_PWM_peri_write(PWM_clk + BCM2835_PWMCLK_DIV, BCM2835_PWM_PASSWRD | (divisor << 12));
  bcm2835_PWM_peri_write(PWM_clk + BCM2835_PWMCLK_CNTL, BCM2835_PWM_PASSWRD | 0x11); // Source=osc and enable
}

//Set the PWM mode
void PWMPi::setMode( uint8_t markspace, uint8_t enabled)
{
  uint32_t control = bcm2835_PWM_peri_read(PWM_pwm + BCM2835_PWM_CONTROL);
  
  if (markspace)
    control |= BCM2835_PWM0_MS_MODE;
  else
    control &= ~BCM2835_PWM0_MS_MODE;
    
  if (enabled)
    control |= BCM2835_PWM0_ENABLE;
  else
    control &= ~BCM2835_PWM0_ENABLE;
    
  // If you use the barrier here, wierd things happen, and the commands dont work
  bcm2835_PWM_peri_write_nb(PWM_pwm + BCM2835_PWM_CONTROL, control);
}

//Set the PWM Range
void PWMPi::setRange(uint32_t range)
{
  bcm2835_PWM_peri_write_nb(PWM_pwm + BCM2835_PWM0_RANGE, range);
}

//Set the PWM Data
void PWMPi::setData(uint32_t data)
{
  bcm2835_PWM_peri_write_nb(PWM_pwm + BCM2835_PWM0_DATA, data);
}

/******************
  Private methods *
 ******************/
 
void *PWMPi::PWM_mapmem(const char *msg, size_t size, int fd, off_t off)
{
  void *map = mmap(NULL, size, (PROT_READ | PROT_WRITE), MAP_SHARED, fd, off);
  if (MAP_FAILED == map)
    fprintf(stderr, "bcm2835_init: %s mmap failed: %s\n", msg, strerror(errno));
  return map;
}

void PWMPi::PWM_unmapmem(void **pmem, size_t size)
{
  if (*pmem == MAP_FAILED) return;
  munmap(*pmem, size);
  *pmem = MAP_FAILED;
}

// Sleep the specified milliseconds
void PWMPi::delay(long millis){
  unistd::usleep(millis*1000);
}

// safe read from peripheral
uint32_t PWMPi::bcm2835_PWM_peri_read(volatile uint32_t* paddr){
  uint32_t ret = *paddr;
  ret = *paddr;
  return ret;
}

// read from peripheral without the read barrier
uint32_t bcm2835_PWM_peri_read_nb(volatile uint32_t* paddr){
    return *paddr;
}

// safe write to peripheral
void PWMPi::bcm2835_PWM_peri_write(volatile uint32_t* paddr, uint32_t value){
  *paddr = value;
  *paddr = value;
}

// write to peripheral without the write barrier
void PWMPi::bcm2835_PWM_peri_write_nb(volatile uint32_t* paddr, uint32_t value){
  *paddr = value;
}

// Set/clear only the bits in value covered by the mask
void PWMPi::bcm2835_PWM_peri_set_bits(volatile uint32_t* paddr, uint32_t value, uint32_t mask){
  uint32_t v = bcm2835_PWM_peri_read(paddr);
  v = (v & ~mask) | (value & mask);
  bcm2835_PWM_peri_write(paddr, v);
}

//
// Low level convenience functions
//

// Function select
// pin is a BCM2835 GPIO pin number NOT RPi pin number
//      There are 6 control registers, each control the functions of a block
//      of 10 pins.
//      Each control register has 10 sets of 3 bits per GPIO pin:
//
//      000 = GPIO Pin X is an input
//      001 = GPIO Pin X is an output
//      100 = GPIO Pin X takes alternate function 0
//      101 = GPIO Pin X takes alternate function 1
//      110 = GPIO Pin X takes alternate function 2
//      111 = GPIO Pin X takes alternate function 3
//      011 = GPIO Pin X takes alternate function 4
//      010 = GPIO Pin X takes alternate function 5
//
// So the 3 bits for port X are:
//      X / 10 + ((X % 10) * 3)
void PWMPi::bcm2835_PWM_gpio_fsel(uint8_t pin, uint8_t mode)
{
  // Function selects are 10 pins per 32 bit word, 3 bits per pin
  volatile uint32_t* paddr = PWM_gpio + BCM2835_PWM_GPFSEL0/4 + (pin/10);
  uint8_t   shift = (pin % 10) * 3;
  uint32_t  mask = BCM2835_PWM_GPIO_FSEL_MASK << shift;
  uint32_t  value = mode << shift;
  bcm2835_PWM_peri_set_bits(paddr, value, mask);
}

PWMPi PWM = PWMPi();